
=begin
/**
  * A class that lets the user draw scribbles with the mouse. The
  * window knows how to redraw itself.
  */
=end

class ScribbleArea < QWidget
=begin
/**
  * The constructor. Initializes the member variables and creates a
  * popup menu that will be popped up when the right mouse button is
  * clicked.
  */
=end
   def initialize
  @_buffer = QPixmap.new(nil)
  @copyBuffer = QPixmap.new(nil)
  @copyRect = QRect.new(0,0,0,0)
  @clipBoard = QApplication::clipboard()
#  // initialize member variables
  @pen = QPen.new(QColor("black"))
  @pen.setStyle(SolidLine)
  @forColor = QColor("black")
  @backColor = QColor("white")
  @fillColor = QColor("white")
  @penWidth = 1
  @mode = MODE_DOT
  @fill = false
#  point of cursor
  @points = Array.new

#  // don't blank the window before repainting
  setBackgroundMode( QWidget::NoBackground )

#  // create a popup menu
  @_popupmenu = QPopupMenu.new
  @_popupmenu.insertItem( i18n("&Clear"), Clear, -1)
  Qt::connect(@_popupmenu,QSIGNAL("activated(int)"),self,"slotClearArea")
# Necessary to catch event functions in Ruby/Qt
  catchEvent
end

=begin

/**
  * This slot sets the current color for the scribble area. It will be
  * connected with the colorChanged( QColor* ) signal from the
  * ScribbleWindow.
  */
=end

def setForColor( new_color )
  @forColor = new_color
end

def setFillColor( new_color )
  @fillColor = new_color
end

def setBackColor( new_color )
  @backColor = new_color
end

def setPenWidth( wd )
   @penWidth = wd
end



def setMode new_mode, fill
   if @mode != new_mode
      @mode = new_mode
      @points.clear
   end
   @fill = fill
end

def drawDot newpos,finish=false
   if @points.size == 0
      @points.push newpos
      return
   end
#  // create a QPainter object for drawing onto the window
  windowpainter = QPainter.new(nil)
#  // and another QPainter object for drawing into an off-screen pixmap
  bufferpainter = QPainter.new(nil)

#  // start painting
  windowpainter.begin( self )    #// self painter paints onto the window
  bufferpainter.begin( @_buffer ) #// and self one in the buffer

#  // set a standard pen with the currently selected color
  @pen.setColor(@forColor)
  @pen.setWidth(@penWidth)
  @pen.setStyle(SolidLine)
  windowpainter.setPen(@pen)
  bufferpainter.setPen(@pen)

#  // draw a line in both the window and the buffer
#   if (!@_last.nil?)
   windowpainter.drawLineQPoint( @points[0], newpos )
   bufferpainter.drawLineQPoint( @points[0], newpos )
#   end
#  // done with painting
  windowpainter.end
  bufferpainter.end

#  // remember the current mouse position
   if finish
     @points.clear
   else
     @points[0] = newpos
   end
end

def drawErase newpos
#  // create a QPainter object for drawing onto the window
  windowpainter = QPainter.new(nil)
#  // and another QPainter object for drawing into an off-screen pixmap
  bufferpainter = QPainter.new(nil)

#  // start painting
  windowpainter.begin( self )    #// self painter paints onto the window
  bufferpainter.begin( @_buffer ) #// and self one in the buffer

#  // set a standard pen with the currently selected color
  @pen.setColor(@backColor)
  @pen.setStyle(SolidLine)
  windowpainter.setPen( @pen )
  bufferpainter.setPen( @pen )
  brush = QBrush.new( @backColor)
  windowpainter.setBrush(brush)
  bufferpainter.setBrush(brush)
#  // draw a line in both the window and the buffer
#   if (!@_last.nil?)
   rec1 = QRect.new(0,0,10,10)
   rec1.moveCenter(newpos)
   windowpainter.drawRectQRect( rec1 )
   bufferpainter.drawRectQRect( rec1 )
#   end
#  // done with painting
  windowpainter.end
  bufferpainter.end
end

def drawLine newpos,finish=false
    self.setCursor( QCursor.new(CrossCursor))
   if @points.size == 0
      @points.push newpos
      return
   end
#  // create a QPainter object for drawing onto the window
  windowpainter = QPainter.new(nil)
#  // and another QPainter object for drawing into an off-screen pixmap
  bufferpainter = QPainter.new(nil)

#  // start painting
  windowpainter.begin( self )    #// self painter paints onto the window
  bufferpainter.begin( @_buffer ) #// and self one in the buffer

#  // set a standard pen with the currently selected color
  windowpainter.setPenQColor( @forColor )
  bufferpainter.setPenQColor( @forColor )

#  // draw a line in both the window and the buffer
   if finish
      @pen.setColor(@forColor)
      @pen.setWidth(@penWidth)
      @pen.setStyle(SolidLine)
      windowpainter.setPen(@pen)
      bufferpainter.setPen(@pen)
      windowpainter.drawLineQPoint( @points[0], newpos )
      bufferpainter.drawLineQPoint( @points[0], newpos )
      @points.clear
      repaintAll
      self.setCursor( QCursor.new(ArrowCursor))
   else
      if @points.size == 1
         windowpainter.drawLineQPoint( @points[0], newpos )
#   bufferpainter.drawLineQPoint( @points[0], newpos )
         @points[1]=newpos
      else

        windowpainter.setPenQColor( @backColor)
        windowpainter.drawLineQPoint( @points[0], @points[1] )
        repaintAll
        windowpainter.setPenQColor( @forColor )
         windowpainter.drawLineQPoint( @points[0], newpos )
        @points[1]=newpos
      end
   end
   windowpainter.end
   bufferpainter.end

end

def drawRect newpos,finish=false
    self.setCursor( QCursor.new(CrossCursor))
   if @points.size == 0
      @points.push newpos
      return
   end
#  // create a QPainter object for drawing onto the window
   windowpainter = QPainter.new(nil)
#  // and another QPainter object for drawing into an off-screen pixmap
   bufferpainter = QPainter.new(nil)

#  // start painting
   windowpainter.begin( self )    #// self painter paints onto the window
   bufferpainter.begin( @_buffer ) #// and self one in the buffer

#  // set a standard pen with the currently selected color
   windowpainter.setPenQColor( @forColor )
   bufferpainter.setPenQColor( @forColor )


#  // draw a line in both the window and the buffer
   if finish
      @pen.setColor(@forColor)
      @pen.setWidth(@penWidth)
      @pen.setStyle(SolidLine)
      windowpainter.setPen(@pen)
      bufferpainter.setPen(@pen)
#   set brush for fill
      if @fill
         brush = QBrush.new( @fillColor)
         windowpainter.setBrush(brush)
         bufferpainter.setBrush(brush)
      end
      windowpainter.drawRect( @points[0].x,@points[0].y,
                                            (newpos.x - @points[0].x ), (newpos.y - @points[0].y ) )
      bufferpainter.drawRect(  @points[0].x,@points[0].y,
                                            (newpos.x - @points[0].x ), (newpos.y - @points[0].y )  )
      @points.clear
      repaintAll
      self.setCursor( QCursor.new(ArrowCursor))
      else
         if @points.size == 1
            windowpainter.drawRect( @points[0].x,@points[0].y,
                                            (newpos.x - @points[0].x ), (newpos.y - @points[0].y ) )
#   bufferpainter.drawLineQPoint( @points[0], newpos )
            @points[1]=newpos
         else

         windowpainter.setPenQColor( @backColor)
         windowpainter.drawRect(  @points[0].x,@points[0].y,
                                            (@points[1].x - @points[0].x ), (@points[1].y - @points[0].y ) )
         repaintAll
         windowpainter.setPenQColor( @forColor )
         windowpainter.drawRect(  @points[0].x,@points[0].y,
                                            (newpos.x - @points[0].x ), (newpos.y - @points[0].y ) )
         @points[1]=newpos
         end
     end
     windowpainter.end
     bufferpainter.end

end

def drawCircle newpos,finish=false
    self.setCursor( QCursor.new(CrossCursor))
   if @points.size == 0
      @points.push newpos
      return
   end
#  // create a QPainter object for drawing onto the window
   windowpainter = QPainter.new(nil)
#  // and another QPainter object for drawing into an off-screen pixmap
   bufferpainter = QPainter.new(nil)

#  // start painting
   windowpainter.begin( self )    #// self painter paints onto the window
   bufferpainter.begin( @_buffer ) #// and self one in the buffer

#  // set a standard pen with the currently selected color
   windowpainter.setPenQColor( @forColor )
   bufferpainter.setPenQColor( @forColor )

#  // draw a line in both the window and the buffer
   if finish
      @pen.setColor(@forColor)
      @pen.setWidth(@penWidth)
      @pen.setStyle(SolidLine)
      windowpainter.setPen(@pen)
      bufferpainter.setPen(@pen)
#   set brush for fill
      if @fill
         brush = QBrush.new( @fillColor)
         windowpainter.setBrush(brush)
         bufferpainter.setBrush(brush)
      end

      windowpainter.drawEllipse( @points[0].x,@points[0].y,
                                            (newpos.x - @points[0].x ), (newpos.y - @points[0].y ) )
      bufferpainter.drawEllipse(  @points[0].x,@points[0].y,
                                            (newpos.x - @points[0].x ), (newpos.y - @points[0].y ) )
      @points.clear
      repaintAll
      self.setCursor( QCursor.new(ArrowCursor))
      else
         if @points.size == 1
            windowpainter.drawEllipse( @points[0].x,@points[0].y,
                                            (newpos.x - @points[0].x ), (newpos.y - @points[0].y ) )
#   bufferpainter.drawLineQPoint( @points[0], newpos )
            @points[1]=newpos
         else

         windowpainter.setPenQColor( @backColor)
         windowpainter.drawEllipse(  @points[0].x,@points[0].y,
                                            (@points[1].x - @points[0].x ), (@points[1].y - @points[0].y ) )
         repaintAll
         windowpainter.setPenQColor( @forColor )
         windowpainter.drawEllipse(  @points[0].x,@points[0].y,
                                            (newpos.x - @points[0].x ), (newpos.y - @points[0].y ) )
         @points[1]=newpos
         end
    end
    windowpainter.end
    bufferpainter.end
end

def selectArea newpos,finish=false
    self.setCursor( QCursor.new(CrossCursor))
   if @points.size == 0
      @points.push newpos
      return
   end
#  // create a QPainter object for drawing onto the window
   windowpainter = QPainter.new(nil)

#  // start painting
   windowpainter.begin( self )    #// self painter paints onto the window

#  // set a standard pen with the currently selected color
   @pen.setColor(@forColor)
   @pen.setStyle(DotLine)
   @pen.setWidth(1)
   windowpainter.setPen( @pen)


#  // draw a line in both the window and the buffer
   if finish and @points.size() > 1
      width = @points[1].x() -@points[0].x()
      height = @points[1].y() - @points[0].y()
      @copyRect.setX(@points[0].x)
      @copyRect.setY(@points[0].y)
      @copyRect.setWidth(width)
      @copyRect.setHeight(height)
      @copyRect = @copyRect.normalize()
      @points.clear
#      repaintAll
      self.setCursor( QCursor.new(ArrowCursor))
   else
       if finish
           self.setCursor( QCursor.new(ArrowCursor))
           @points.clear

       elsif @points.size == 1
          windowpainter.drawRect( @points[0].x,@points[0].y,
                                            (newpos.x - @points[0].x ), (newpos.y - @points[0].y ) )
#   bufferpainter.drawLineQPoint( @points[0], newpos )
          @points[1]=newpos
       else
            @pen.setColor(@backColor)
            @pen.setStyle(DotLine)
            windowpainter.setPen(@pen)
            windowpainter.drawRect(  @points[0].x,@points[0].y,
                                            (@points[1].x - @points[0].x ), (@points[1].y - @points[0].y ) )
            repaintAll
            @pen.setColor(@forColor)
            @pen.setStyle(DotLine)
            windowpainter.setPen( @pen )
            windowpainter.drawRect(  @points[0].x,@points[0].y,
                                            (newpos.x - @points[0].x ), (newpos.y - @points[0].y ) )
            @points[1]=newpos
         end
    end
   windowpainter.end

end


=begin
/**
  * This slot clears the scribble area by filling the off-screen buffer with
  * white and copying it over to the window.
  */
=end
def slotClearArea dum=0

#  // fill the off-screen buffer with plain white
  @_buffer.fill( @backColor )

#  // and copy it over to the window
  bitBlt( self, 0, 0, @_buffer )
end

=begin
/**
  * This method does the actual loading. It relies on QPixmap (and the
  * underlying I/O machinery) to determine the filetype.
  */
=end

def slotLoad( filename )

  if( !@_buffer.load( filename ) )
	QMessageBox::warning( nil, i18n("Load error"), i18n("Could not load file"),1 ,0)
  end
  self.resizeQSize(@_buffer.size)
  repaintAll
end

=begin
/**
  * This method does the actual saving. We hardcode the file type as
  * BMP. Unix users might want to replace self with something more usual
  * on Unix like XPM.
  */
=end

def slotSave( filename )
=begin
    QImageIO iio;
    QImage   im;
    im = pixmap;                                // convert to image
    iio.setImage( im );
    iio.setFileName( "burger.bmp" );
    iio.setFormat( "BMP" );  // JPEG,GIF,XPM,BMP,PNG etc.
    iio.write();                                // TRUE if ok
=end
  fname = filename.split('.')
#print "fname[1] = #{fname[1]}\n"
  ext = fname[1].upcase

  if ext == ""
      ext = "XPM"
  elsif ext == "JPG"
      ext = "JPEG"
  end
#print "filename is #{filename}\n"
#print "File Format is #{ext} for save.\n"
  if( !@_buffer.save( filename, ext) )
	QMessageBox::warning( 0, i18n("Save error"), i18n("Could not save file") )
  end
end

def getPixmap
   @_buffer
end

=begin
/**
  * This virtual method gets called whenever the users presses the
  * mouse over the window. When the right mouse button is pressed, it
  * pops up synchronously a previously constructed popup
  * menu. Otherwise, it just records the position of the mouse
  * at the time of the click.
  */
=end
def mousePressEvent(  event )
  newpos = QPoint.new(event.pos.x,event.pos.y) #necessary to deepcopy.
  if( event.button == RightButton )
	@_popupmenu.execQPoint( QCursor::pos )
   else
      case @mode
      when MODE_PASTE
#         @mode = MODE_PASTE_END
      when MODE_SELECT
         selectArea newpos
      when MODE_DOT
         drawDot newpos
      when MODE_LINE
         drawLine newpos
      when MODE_RECT
         drawRect newpos
      when MODE_CIRCLE
         drawCircle newpos
      when MODE_ERASE
         drawErase newpos
      end
   end
end

=begin
/**
  * This virtual method gets called whenever the user moves the mouse
  * while the mouse button is pressed (self is also known as
  * "dragging"). If we had called setMouseTracking( true ) before,
  * self method would also be called when the mouse was moved without
  * any button pressed. We know that we haven't, and thus don't have
  * to check whether any buttons are pressed.
  */
=end

def mouseMoveEvent( event )
  newpos = QPoint.new(event.pos.x,event.pos.y)
   case @mode
   when MODE_SELECT
      selectArea newpos
   when MODE_DOT
      drawDot newpos
   when MODE_LINE
      drawLine newpos
   when MODE_RECT
      drawRect newpos
   when MODE_CIRCLE
      drawCircle newpos
   when MODE_ERASE
      drawErase newpos
   end
end

def mouseReleaseEvent( event )
  newpos = QPoint.new(event.pos.x,event.pos.y) #necessary to deepcopy.
  if( event.button == LeftButton )
      case @mode
      when MODE_PASTE
         @mode = MODE_PASTE_END
      when MODE_SELECT
         selectArea newpos, true
      when MODE_DOT
         drawDot newpos, true
      when MODE_LINE
         drawLine newpos, true
      when MODE_RECT
         drawRect newpos, true
      when MODE_CIRCLE
         drawCircle newpos, true
      end
   end
end


=begin
/**
  *	This virtual method gets called whenever the widget needs
  * painting, e.g., when it has been obscured and unhidden again.
  */
=end
def paintEvent( event )

#  // copy the image from the buffer pixmap to the window
  bitBlt( self, 0, 0, @_buffer )
end

=begin
/**
  * This virtual method gets called whenever the window is resized. We
  * use it to make sure that the off-screen buffer always has the same
  * size as the window.
  * In order not to lose the original scribbling, it is first copied
  * to a temporary buffer. After the main buffer has been resized and
  * filled with white, the image is copied from the temporary buffer to
  * the main buffer.
  */
=end
def resizeEvent( event )

  save=QPixmap.new( @_buffer )
  @_buffer.resizeQSize( event.size )
  @_buffer.fill( @backColor)
  bitBlt( @_buffer, 0, 0, save )
end

def slotCut
     repaintAll
     @copyBuffer.resizeQSize(@copyRect.size())
     bitBlt(@copyBuffer,0,0,@_buffer,@copyRect.x(),@copyRect.y(),@copyRect.width(),@copyRect.height())
     #@clipBoard.setPixmap(@copyBuffer) # not supported yet.
     p = QPainter.new(self)
     pb = QPainter.new(@_buffer)
     p.setPenQColor(@backColor)
     pb.setPenQColor(@backColor)
     brush = QBrush.new(@backColor)
     p.setBrush(brush)
     pb.setBrush(brush)
     p.drawRectQRect(@copyRect)
     pb.drawRectQRect(@copyRect)
     p.end
     pb.end
end
def slotCopy
     repaintAll
     @copyBuffer.resizeQSize(@copyRect.size())
     bitBlt(@copyBuffer,0,0,@_buffer,@copyRect.x(),@copyRect.y(),@copyRect.width(),@copyRect.height())
     #@clipBoard.setPixmap(@copyBuffer) #not supported yet
end
def slotPaste
    repaintAll
    if @copyBuffer.width() != 0 and @copyBuffer.height() !=0
      @mode = MODE_PASTE
      self.setCursor( QCursor.new(CrossCursor))
      backupPixmap = QPixmap.new(@copyBuffer)
      @cx = 0
      @cy = 0
      bitBlt(backupPixmap,0,0,self,@cx,@cy)
      bitBlt(self,@cx,@cy,@copyBuffer,0,0)
       while (@mode == MODE_PASTE)
       cp = QCursor.pos()
       cp = mapFromGlobal(cp)
       if (@cx != cp.x() or @cy != cp.y())
          bitBlt(self,@cx,@cy,backupPixmap,0,0)
          @cx = cp.x()
          @cy = cp.y()
          bitBlt(backupPixmap,0,0,self,@cx,@cy)
          bitBlt(self,@cx,@cy,@copyBuffer,0,0)
          end
          QApplication::qApp.processEvents()
       end
       self.setCursor( QCursor.new(ArrowCursor))
       bitBlt(@_buffer,@cx,@cy,@copyBuffer,0,0)
       @mode = MODE_SELECT
        repaintAll
    end
end
end
