
class ScribbleWindow < KTMainWindow

def initialize
   kl = KIconLoader.new
   @fileOpenText = i18n("Click this button to open a new file.\n\nYou can also select the Open command from the File menu.")
   @fileSaveText = i18n("Click this button to save the file you are editing.  You will be prompted for a file name.\n\nYou can also select the Save command from the File menu.\n")
   @filePrintText = i18n("Click this button to print the file you are editing.\n\nYou can also select the Print command from the File menu.")

   @newIcon = kl.loadIcon(QString.new("filenew.xpm"))
   @openIcon = kl.loadIcon(QString.new("fileopen.xpm"))
   @saveIcon = kl.loadIcon(QString.new("filefloppy.xpm"))
   @printIcon = kl.loadIcon(QString.new("fileprint.xpm"))
   @copyIcon = kl.loadIcon(QString.new("editcopy.xpm"))
   @copyCut = kl.loadIcon(QString.new("editcut.xpm"))
   @copyPaste = kl.loadIcon(QString.new("editpaste.xpm"))
   @arrowIcon = kl.loadIcon(QString.new("arrow.xpm"))
   @penIcon = kl.loadIcon(QString.new("pen.xpm"))
   @lineIcon = kl.loadIcon(QString.new("line.xpm"))
   @circleIcon = kl.loadIcon(QString.new("circle.xpm"))
   @rectIcon = kl.loadIcon(QString.new("toolrect.xpm"))
   @eraseIcon = kl.loadIcon(QString.new("erase.xpm"))
   @fillIcon = kl.loadIcon(QString.new("circle_fill.xpm"))

   @forColorChanged = RSignal.new("Object")
   @fillColorChanged = RSignal.new("Object")
   @backColorChanged = RSignal.new("Object")
   
   @save = RSignal.new("const char*")
   @load = RSignal.new("const char*")

   @fileTools = KToolBar.new( self, "file operations" )

   @fileTools.insertButton( @newIcon, Clear,true,i18n("New"))
   @fileTools.insertButton( @openIcon, Load,true,i18n("Load"))
   @fileTools.insertButton( @saveIcon, Save,true,i18n("Save"))
   @fileTools.insertButton( @printIcon, Print,true,i18n("Print"))
   @fileTools.insertSeparator
   @fileTools.insertButton( @copyIcon, Copy,true,i18n("Copy"))
   @fileTools.insertButton( @copyCut, Cut,true,i18n("Cut"))
   @fileTools.insertButton( @copyPaste, Paste,true,i18n("Paste"))
   @fileTools.insertSeparator
   @fileTools.insertButton( kl.loadIcon(QString.new("exit.xpm")),Quit,true,i18n("Quit"))
   @whatsThisButton = QWhatsThis.whatsThisButton(@fileTools)
   @fileTools.insertWidget(999,@whatsThisButton.sizeHint.width,@whatsThisButton)

   QWhatsThis::add( @fileTools.getWidget(Load), @fileOpenText, false )
   QWhatsThis::add( @fileTools.getWidget(Save), @fileSaveText, false )
   QWhatsThis::add( @fileTools.getWidget(Print), @filePrintText, false )
   
   addToolBar(@fileTools)
   Qt::connect(@fileTools,QSIGNAL("clicked(int)"),self,"menuActivated")
   
   @drawTools = KToolBar.new(self, "draw operations")
   @drawTools.insertButton( @arrowIcon, MODE_SELECT,true,i18n("Select"))
   @drawTools.setToggle(MODE_SELECT,true)
   @drawTools.toggleButton(MODE_SELECT)
   @drawTools.insertButton( @penIcon, MODE_DOT,true,i18n("Draw"))
   @drawTools.setToggle(MODE_DOT,true)
   @drawTools.insertButton( @lineIcon, MODE_LINE,true,i18n("Line"))
   @drawTools.setToggle(MODE_LINE,true)
   @drawTools.insertButton( @circleIcon, MODE_CIRCLE,true,i18n("Circle"))
   @drawTools.setToggle(MODE_CIRCLE,true)
   @drawTools.insertButton( @rectIcon, MODE_RECT,true,i18n("Rectangle"))
   @drawTools.setToggle(MODE_RECT,true)
   @drawTools.insertButton( @eraseIcon, MODE_ERASE,true,i18n("Erase"))
   @drawTools.setToggle(MODE_ERASE,true)
   
   @drawTools.insertSeparator
   
   @kcButtonLine = KColorButton.new(QColor("black"),@drawTools)
   @kcButtonLine.setText("P")
   @drawTools.insertWidget(COLOR_PEN,30,@kcButtonLine)
   QToolTip::add( @kcButtonLine, i18n("Line Color") )
   Qt::connect(@kcButtonLine,QSIGNAL("changed(const QColor &)"),
         self, "setForQColor")
   Qt::connect(@forColorChanged,@kcButtonLine,"setColor")

   @kcButtonFill = KColorButton.new(QColor("white"),@drawTools)
   @kcButtonFill.setText("F")
   @drawTools.insertWidget(COLOR_FILL,30,@kcButtonFill)
   QToolTip::add( @kcButtonFill, i18n("Fill Color") );
   Qt::connect(@kcButtonFill,QSIGNAL("changed(const QColor &)"),
         self, "setFillQColor")
   Qt::connect(@fillColorChanged,@kcButtonFill,"setColor")

           
   @kcButtonBack = KColorButton.new(QColor("white"),@drawTools)
   @kcButtonBack.setText("B")
   @drawTools.insertWidget(COLOR_BACK,30,@kcButtonBack)
   QToolTip::add( @kcButtonBack, i18n("Background Color") );
   Qt::connect(@kcButtonBack,QSIGNAL("changed(const QColor &)"),
         self, "setBackQColor")  
   Qt::connect(@backColorChanged,@kcButtonBack,"setColor")

   @drawTools.insertSeparator
   
   @drawTools.insertButton( @fillIcon, MODE_FILL,true,i18n("Fill Object"))
   @drawTools.setToggle(MODE_FILL,true)   
   
   Qt::connect(@drawTools,QSIGNAL("clicked(int)"),self,"drawToolsActivated")
   
   addToolBar(@drawTools)

   #/* The next few lines build up the menubar. We first create the menus
   #* one by one and add them afterwards to the menubar. */

   @_filemenu = QPopupMenu.new   #// create a file menu
   @_filemenu.insertItemQPixmapTextId(@openIcon, i18n("&New"), Clear)
   @_filemenu.setAccel(Qt::CTRL+Key_N,Clear)
   @_filemenu.insertItemQPixmapTextId(@openIcon, i18n("&Open"), Load)
   @_filemenu.setAccel(Qt::CTRL+Key_L,Load)
   @_filemenu.insertItemQPixmapTextId(@saveIcon, i18n("&Save"), Save)
   @_filemenu.setAccel(Qt::CTRL+Key_S,Save)
   @_filemenu.insertSeparator
   @_filemenu.insertItemQPixmapTextId(@printIcon,i18n("Print"),Print)
   @_filemenu.setAccel(Qt::CTRL+Key_P,Print)
   @_filemenu.insertSeparator
   @_filemenu.insertItemQObj( i18n("&Quit"), QApplication::qApp, QSLOT( "quit()" ) )
   Qt::connect(@_filemenu, QSIGNAL( "activated( int )"),
					self, "menuActivated")

   @_editmenu = QPopupMenu.new   #// create a file menu
   @_editmenu.insertItemQPixmapTextId(@openIcon, i18n("&Cut"), Cut)
   @_editmenu.setAccel(Qt::CTRL+Key_C,Clear)
   @_editmenu.insertItemQPixmapTextId(@openIcon, i18n("C&opy"), Copy)
   @_editmenu.setAccel(Qt::CTRL+Key_O,Load)
   @_editmenu.insertItemQPixmapTextId(@saveIcon, i18n("&Paste"), Paste)
   @_editmenu.setAccel(Qt::CTRL+Key_P,Save)

   Qt::connect(@_editmenu, QSIGNAL( "activated( int )"),
					self, "menuActivated")

   @_toolsmenu = QPopupMenu.new   #// create a file menu
   @_toolsmenu.setCheckable(true)
   @menuSelect = @_toolsmenu.insertItem(i18n("&Select"), MODE_SELECT)
   @menuDraw = @_toolsmenu.insertItem(i18n("&Draw"), MODE_DOT)
   @_toolsmenu.setAccel(Qt::CTRL+Key_D,MODE_DOT)
   @menuLine = @_toolsmenu.insertItem( i18n("&Line"), MODE_LINE)
   @_toolsmenu.setAccel(Qt::CTRL+Key_L,MODE_LINE)
   @menuCircle = @_toolsmenu.insertItem(i18n("&CIrcle"), MODE_CIRCLE)
   @_toolsmenu.setAccel(Qt::CTRL+Key_S,MODE_CIRCLE)
   @menuRect = @_toolsmenu.insertItem( i18n("&Rectangle"), MODE_RECT)
   @_toolsmenu.setAccel(Qt::CTRL+Key_R,MODE_RECT)
   @menuErase = @_toolsmenu.insertItem( i18n("&Erase"), MODE_ERASE)
   @_toolsmenu.setAccel(Qt::CTRL+Key_R,MODE_ERASE)

   @_toolsmenu.insertSeparator
   @menuFill = @_toolsmenu.insertItem( i18n("&Fill"), MODE_FILL)
   @_toolsmenu.setAccel(Qt::CTRL+Key_F,MODE_FILL)
   @mode = MODE_DOT
   @fill = false
   setToolsMenuCheck @mode, @fill

   Qt::connect(@_toolsmenu, QSIGNAL( "activated( int )"),
					self, "modeMenuActivated")
   @penmenu = QPopupMenu.new
   @penmenu.setCheckable(true)
   @menuw0 = @penmenu.insertItem(i18n("width 0"),PEN_WIDTH_0)
   @menuw1 = @penmenu.insertItem(i18n("width 1"),PEN_WIDTH_1)
   @menuw2 = @penmenu.insertItem(i18n("width 2"),PEN_WIDTH_2)
   @menuw4 = @penmenu.insertItem(i18n("width 4"),PEN_WIDTH_4)
   @menuw8 = @penmenu.insertItem(i18n("width 8"),PEN_WIDTH_8)
   
    
   @forColormenu = QPopupMenu.new   #// create a color menu
   @forColormenu.insertItem( i18n("B&lack"), COLOR_MENU_ID_BLACK )
   @forColormenu.insertItem( i18n("&Red"), COLOR_MENU_ID_RED )
   @forColormenu.insertItem( i18n("&Blue"), COLOR_MENU_ID_BLUE )
   @forColormenu.insertItem( i18n("&Green"), COLOR_MENU_ID_GREEN )
   @forColormenu.insertItem( i18n("&Yellow"), COLOR_MENU_ID_YELLOW )
   @forColormenu.insertItem( i18n("&White"), COLOR_MENU_ID_WHITE )
   @forColormenu.insertItem( i18n("&Custum..."),COLOR_MENU_ID_CUSTUM)
   Qt::connect( @forColormenu, QSIGNAL( "activated( int )" ),
					self, "slotForColorMenu")
   @filColormenu = QPopupMenu.new   #// create a color menu
   @filColormenu.insertItem( i18n("B&lack"), COLOR_MENU_ID_BLACK )
   @filColormenu.insertItem( i18n("&Red"), COLOR_MENU_ID_RED )
   @filColormenu.insertItem( i18n("&Blue"), COLOR_MENU_ID_BLUE )
   @filColormenu.insertItem( i18n("&Green"), COLOR_MENU_ID_GREEN )
   @filColormenu.insertItem( i18n("&Yellow"), COLOR_MENU_ID_YELLOW )
   @filColormenu.insertItem( i18n("&White"), COLOR_MENU_ID_WHITE )
   @filColormenu.insertItem( i18n("&Custum..."),COLOR_MENU_ID_CUSTUM)
   Qt::connect( @filColormenu, QSIGNAL( "activated( int )" ),
					self, "slotFillColorMenu"  )  
   @backColormenu = QPopupMenu.new   #// create a color menu
   @backColormenu.insertItem( i18n("B&lack"), COLOR_MENU_ID_BLACK )
   @backColormenu.insertItem( i18n("&Red"), COLOR_MENU_ID_RED )
   @backColormenu.insertItem( i18n("&Blue"), COLOR_MENU_ID_BLUE )
   @backColormenu.insertItem( i18n("&Green"), COLOR_MENU_ID_GREEN )
   @backColormenu.insertItem( i18n("&Yellow"), COLOR_MENU_ID_YELLOW )
   @backColormenu.insertItem( i18n("&White"), COLOR_MENU_ID_WHITE )
   @backColormenu.insertItem( i18n("&Custum..."),COLOR_MENU_ID_CUSTUM)
   Qt::connect( @backColormenu, QSIGNAL( "activated( int )" ),
					self, "slotBackColorMenu"  )
   @_colormenu = QPopupMenu.new #// create a color menu
   @_colormenu.insertItemQPopupMenu(i18n("Forground Color"),@forColormenu)
   @_colormenu.insertItemQPopupMenu(i18n("Fill Color"),@filColormenu)
   @_colormenu.insertItemQPopupMenu(i18n("Background Color"),@backColormenu)
 													

					

   @_helpmenu = QPopupMenu.new    #// create a help menu
   @_helpmenu.insertItem( i18n("&About Kde Scribble"), About)
   @_helpmenu.insertItem( i18n("About &Qt"), AboutQt)
   Qt::connect(@_helpmenu, QSIGNAL( "activated( int )"),
					self, "menuActivated")
# // create a menu bar
   menuBar.insertItem( i18n("&File"), @_filemenu )
   menuBar.insertItem(i18n("&Edit"),@_editmenu)
   menuBar.insertItem(i18n("&Tools"),@_toolsmenu)
   menuBar.insertItem( i18n("&Color"), @_colormenu )
   menuBar.insertItem(i18n("&Pen"), @penmenu )
   menuBar.insertSeparator
   menuBar.insertItem( i18n("&Help"), @_helpmenu )

   #/* We create a QScrollView and a ScribbleArea. The ScribbleArea will
   #* be managed by the scroll view.*/
   @_scrollview = QScrollView.new( self )
#   @_scrollview.setGeometry( 0, menuBar.height,width, height - menuBar.height )
   @_scribblearea = ScribbleArea.new
   @_scribblearea.setGeometry( 0, 0, 800, 600 )
   @_scrollview.addChild( @_scribblearea )
   
   Qt::connect( @forColorChanged,
                 @_scribblearea, "setForColor"  )
   Qt::connect( @fillColorChanged,
                 @_scribblearea, "setFillColor"  )
   Qt::connect( @backColorChanged,
                 @_scribblearea, "setBackColor"  )

   Qt::connect( @save,
		@_scribblearea, "slotSave"  )
   Qt::connect( @load,
		@_scribblearea, "slotLoad" )
   Qt::connect(@penmenu,QSIGNAL( "activated( int )"),
                @_scribblearea, "setPenWidth")   					

   setView(@_scrollview)

   statusBar.message( i18n("Ready"), 5000 )
   catchEvent
end

def setToolsMenuCheck mode, fill

   case mode
   when MODE_SELECT
   @_toolsmenu.setItemChecked(@menuSelect, true)
   @_toolsmenu.setItemChecked(@menuDraw, false)
   @_toolsmenu.setItemChecked(@menuLine, false)
   @_toolsmenu.setItemChecked(@menuCircle, false)
   @_toolsmenu.setItemChecked(@menuRect, false)
   @_toolsmenu.setItemChecked(@menuErase, false) 
   @drawTools.setButton(MODE_SELECT,true)   
   @drawTools.setButton(MODE_DOT,false)
   @drawTools.setButton(MODE_LINE,false)
   @drawTools.setButton(MODE_CIRCLE,false)
   @drawTools.setButton(MODE_RECT,false)
   @drawTools.setButton(MODE_ERASE,false)
   
    
   when MODE_DOT
   @_toolsmenu.setItemChecked(@menuSelect, false)
   @_toolsmenu.setItemChecked(@menuDraw, true)
   @_toolsmenu.setItemChecked(@menuLine, false)
   @_toolsmenu.setItemChecked(@menuCircle, false)
   @_toolsmenu.setItemChecked(@menuRect, false)
   @_toolsmenu.setItemChecked(@menuErase, false)
   @drawTools.setButton(MODE_SELECT,false)
   @drawTools.setButton(MODE_DOT,true)
   @drawTools.setButton(MODE_LINE,false)
   @drawTools.setButton(MODE_CIRCLE,false)
   @drawTools.setButton(MODE_RECT,false)
   @drawTools.setButton(MODE_ERASE,false)

   when MODE_LINE
   @_toolsmenu.setItemChecked(@menuSelect, false)
   @_toolsmenu.setItemChecked(@menuDraw, false)
   @_toolsmenu.setItemChecked(@menuLine, true)
   @_toolsmenu.setItemChecked(@menuCircle, false)
   @_toolsmenu.setItemChecked(@menuRect, false)
   @_toolsmenu.setItemChecked(@menuErase, false)
   @drawTools.setButton(MODE_SELECT,false)
   @drawTools.setButton(MODE_DOT,false)
   @drawTools.setButton(MODE_LINE,true)   
   @drawTools.setButton(MODE_CIRCLE,false)
   @drawTools.setButton(MODE_RECT,false)
   @drawTools.setButton(MODE_ERASE,false)

   when MODE_CIRCLE
   @_toolsmenu.setItemChecked(@menuSelect, false)
   @_toolsmenu.setItemChecked(@menuDraw, false)
   @_toolsmenu.setItemChecked(@menuLine, false)
   @_toolsmenu.setItemChecked(@menuCircle, true)
   @_toolsmenu.setItemChecked(@menuRect, false)
   @_toolsmenu.setItemChecked(@menuErase, false)
   @drawTools.setButton(MODE_SELECT,false)
   @drawTools.setButton(MODE_DOT,false)
   @drawTools.setButton(MODE_LINE,false)
   @drawTools.setButton(MODE_CIRCLE,true) 
   @drawTools.setButton(MODE_RECT,false)
   @drawTools.setButton(MODE_ERASE,false)

   when MODE_RECT
   @_toolsmenu.setItemChecked(@menuSelect, false)
   @_toolsmenu.setItemChecked(@menuDraw, false)
   @_toolsmenu.setItemChecked(@menuLine, false)
   @_toolsmenu.setItemChecked(@menuCircle, false)
   @_toolsmenu.setItemChecked(@menuRect, true)
   @_toolsmenu.setItemChecked(@menuErase, false)
   @drawTools.setButton(MODE_SELECT,false)
   @drawTools.setButton(MODE_DOT,false)
   @drawTools.setButton(MODE_LINE,false)
   @drawTools.setButton(MODE_CIRCLE,false)
   @drawTools.setButton(MODE_RECT,true) 
   @drawTools.setButton(MODE_ERASE,false)
   
  when MODE_ERASE
   @_toolsmenu.setItemChecked(@menuSelect, false)
   @_toolsmenu.setItemChecked(@menuDraw, false)
   @_toolsmenu.setItemChecked(@menuLine, false)
   @_toolsmenu.setItemChecked(@menuCircle, false)
   @_toolsmenu.setItemChecked(@menuRect, false)
   @_toolsmenu.setItemChecked(@menuErase, true)
   @drawTools.setButton(MODE_SELECT,false)
   @drawTools.setButton(MODE_DOT,false)
   @drawTools.setButton(MODE_LINE,false)
   @drawTools.setButton(MODE_CIRCLE,false)
   @drawTools.setButton(MODE_RECT,false)
   @drawTools.setButton(MODE_ERASE,true)

   end
   if fill
      @_toolsmenu.setItemChecked(@menuFill, true)
      @drawTools.setButton(MODE_FILL,true)
   else
      @_toolsmenu.setItemChecked(@menuFill, false)
      @drawTools.setButton(MODE_FILL,false)
   end

end

def slotAbout

  QMessageBox::informationOK( self, i18n("About Kde Scribble 5"),
			i18n("This is the Scribble application\noriginaly written by Matthias Kalle Dalheimer\nin \"Programming with Qt\" by O'Relilly & Assoc. Inc.\n(C) 2000 translated to Ruby/Kde by Nobuyuki Horie\n"))
GC.start
end


def slotAboutQt

  QMessageBox::aboutQt( self, i18n("About Qt") )
end

def helpMenuActivated( item )
   case item
     when About
       slotAbout
     when AboutQt
       slotAboutQt
   end
end

def setForQColor qcolor
   @forColorChanged.sendWith(qcolor)
end

def setFillQColor qcolor
   @fillColorChanged.sendWith(qcolor)
end

def setBackQColor qcolor
   @backColorChanged.sendWith(qcolor)
end


def slotForColorMenu id
   setColor @forColorChanged, id
end

def slotFillColorMenu id
   setColor @fillColorChanged, id
end

def slotBackColorMenu id
   setColor @backColorChanged, id
end



def setColor colorSignal,id

  case id
	when COLOR_MENU_ID_BLACK
	   colorSignal.sendWith( QColor("black") )	
	when COLOR_MENU_ID_RED
	   colorSignal.sendWith( QColor("red") )
	when COLOR_MENU_ID_BLUE
	   colorSignal.sendWith( QColor("blue") )
	when COLOR_MENU_ID_GREEN
	   colorSignal.sendWith( QColor("green") )
	when COLOR_MENU_ID_YELLOW
	   colorSignal.sendWith( QColor("yellow") )
	when COLOR_MENU_ID_WHITE
	   colorSignal.sendWith( QColor("white") )
        when COLOR_MENU_ID_CUSTUM
            colorSignal.sendWith( KColorDialog::getColor()[1])
	end
end

=begin
/**
  * This is the slot for the menu item File/Load. It opens a
  * QFileDialog to ask the user for a filename and emits a save()
  * signal with the filename as a parameter afterwards.
  */
=end

def slotLoad

  #/* Open a file dialog for loading. The default directory is the
   #* current directory, the filter *.bmp.
   #*/
  fd = QFileDialog.new(".","*.xpm",nil,"fd",true)
  fd.setMode( QFileDialog::ExistingFile )
  ql =QStrList.new
  ql.append("*.xpm")
  ql.append("*.bmp")
  ql.append("*.gif")
  ql.append("*.jpg")
  ql.append("*.png")
  ql.append("*.*")
  fd.setFiltersQStrList(ql)
  fd.show
  filename = fd.selectedFile
  if( !filename.isEmpty )
	@load.sendWith( filename.to_str )
  end
end

=begin
/**
  * This is the save equivalent to slotLoad(). Again we just ask for a
  * filename and emit a signal.
  */
=end
def slotSave

  #/* Open a file dialog for saving. The default directory is the
   #* current directory, the filter *.bmp.
   #*/
  fd = QFileDialog.new( ".", "*.xpm", nil, "fd",true )
  fd.setMode(QFileDialog::AnyFile)
  ql =QStrList.new
  ql.append("*.xpm")
  ql.append("*.bmp")
  ql.append("*.jpg")
  ql.append("*.png")
  ql.append("*.*")
  fd.setFiltersQStrList(ql)
  fd.show
  filename = fd.selectedFile

  if( !filename.isEmpty  )
	@save.sendWith( filename.to_str )
  end
end

def menuActivated id
   case id
   when Load
     slotLoad
   when Save
     slotSave
   when Print
     slotPrint
   when Clear
     @_scribblearea.slotClearArea
    when Cut
     @_scribblearea.slotCut
    when Copy
     @_scribblearea.slotCopy
    when Paste
     @_scribblearea.slotPaste
  when About
     slotAbout
   when AboutQt
     slotAboutQt
   when Quit
     QApplication::qApp.quit
   end
end

def modeMenuActivated id
   if id ==MODE_FILL
      if @fill
         @fill = false
      else
         @fill = true
      end
   else
     @mode = id
   end
   @_scribblearea.setMode @mode, @fill
   setToolsMenuCheck @mode, @fill
end

def drawToolsActivated id
   modeMenuActivated id
end

def slotPrint
    printer = QPrinter.new
    printer.setMinMax( 1, 10 );

   if ( printer.setup(self) ) 		#// printer dialog
	self.statusBar.message( i18n("Printing...") ,4000)
	p = QPainter.new(nil)
	p.begin( printer )			#// paint on printer
          p.drawPixmap(10,10, @_scribblearea.getPixmap)

	p.end				#// send job to printer
	self.statusBar.message( i18n("Printing completed"), 2000 )
   else
	self.statusBar.message( i18n("Printing aborted"), 2000 )
   end

end

end
